@@ -34,9 +34,10 @@
 ///        each buffer swap.
 
 #define _POSIX_C_SOURCE 199309L // glibc feature macro for nanosleep.
-#define WAFFLE_API_VERSION 0x0103
+#define WAFFLE_API_VERSION 0x0106
 #define WAFFLE_API_EXPERIMENTAL
 
+#include <assert.h>
 #include <getopt.h>
 #include <stdarg.h>
 #include <stdbool.h>
@@ -51,7 +52,7 @@
 
 #ifdef __APPLE__
 #    import <Foundation/NSAutoreleasePool.h>
-#    import <Appkit/NSApplication.h>
+#    import <AppKit/NSApplication.h>
 
 static void
 removeXcodeArgs(int *argc, char **argv);
@@ -61,14 +62,15 @@ removeXcodeArgs(int *argc, char **argv);
 
 static const char *usage_message =
     "usage:\n"
-    "    gl_basic --platform=android|cgl|gbm|glx|wayland|wgl|x11_egl\n"
+    "    gl_basic --platform=android|cgl|gbm|glx|wayland|wgl|surfaceless_egl|x11_egl\n"
     "             --api=gl|gles1|gles2|gles3\n"
     "             [--version=MAJOR.MINOR]\n"
     "             [--profile=core|compat|none]\n"
     "             [--forward-compatible]\n"
     "             [--debug]\n"
+    "             [--robust]\n"
     "             [--resize-window]\n"
-    "             [--window-size=WIDTHxHEIGHT]\n"
+    "             [--window-size=WIDTHxHEIGHT | --fullscreen]\n"
     "\n"
     "examples:\n"
     "    gl_basic --platform=glx --api=gl\n"
@@ -85,8 +87,14 @@ static const char *usage_message =
     "    --debug\n"
     "        Create a debug context.\n"
     "\n"
+    "    --robust\n"
+    "        Create a robust context.\n"
+    "\n"
     "    --resize-window\n"
     "        Resize the window between each draw call.\n"
+    "\n"
+    "    --fullscreen\n"
+    "        Create a fullscreen window.\n"
     ;
 
 enum {
@@ -96,8 +104,10 @@ enum {
     OPT_PROFILE,
     OPT_DEBUG,
     OPT_FORWARD_COMPATIBLE,
+    OPT_ROBUST,
     OPT_RESIZE_WINDOW,
     OPT_WINDOW_SIZE,
+    OPT_FULLSCREEN,
 };
 
 static const struct option get_opts[] = {
@@ -107,8 +117,10 @@ static const struct option get_opts[] = {
     { .name = "profile",        .has_arg = required_argument,     .val = OPT_PROFILE },
     { .name = "debug",          .has_arg = no_argument,           .val = OPT_DEBUG },
     { .name = "forward-compatible", .has_arg = no_argument,       .val = OPT_FORWARD_COMPATIBLE },
+    { .name = "robust",         .has_arg = no_argument,           .val = OPT_ROBUST },
     { .name = "resize-window",  .has_arg = no_argument,           .val = OPT_RESIZE_WINDOW },
     { .name = "window-size",    .has_arg = required_argument,     .val = OPT_WINDOW_SIZE },
+    { .name = "fullscreen",     .has_arg = no_argument,           .val = OPT_FULLSCREEN },
     { 0 },
 };
 
@@ -191,6 +203,7 @@ enum {
     GL_CONTEXT_FLAGS = 0x821e,
     GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT = 0x00000001,
     GL_CONTEXT_FLAG_DEBUG_BIT              = 0x00000002,
+    GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT      = 0x00000004 ,
 };
 
 static int window_width = 320;
@@ -206,6 +219,7 @@ static int window_height = 240;
 
 static void (APIENTRY *glClearColor)(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
 static void (APIENTRY *glClear)(GLbitfield mask);
+static GLenum (APIENTRY *glGetError)(void);
 static void (APIENTRY *glGetIntegerv)(GLenum pname, GLint *params);
 static void (APIENTRY *glReadPixels)(GLint x, GLint y, GLsizei width, GLsizei height,
                                      GLenum format, GLenum type, GLvoid* data);
@@ -226,9 +240,12 @@ struct options {
 
     bool context_forward_compatible;
     bool context_debug;
+    bool context_robust;
 
     bool resize_window;
 
+    bool fullscreen;
+
     /// @brief One of `WAFFLE_DL_*`.
     int dl;
 };
@@ -245,6 +262,7 @@ static const struct enum_map platform_map[] = {
     {WAFFLE_PLATFORM_GLX,       "glx"           },
     {WAFFLE_PLATFORM_WAYLAND,   "wayland"       },
     {WAFFLE_PLATFORM_WGL,       "wgl"           },
+    {WAFFLE_PLATFORM_SURFACELESS_EGL,   "surfaceless_egl"       },
     {WAFFLE_PLATFORM_X11_EGL,   "x11_egl"       },
     {0,                         0               },
 };
@@ -284,6 +302,7 @@ parse_args(int argc, char *argv[], struct options *opts)
 {
     bool ok;
     bool loop_get_opt = true;
+    bool found_window_size = false;
 
 #ifdef __APPLE__
     removeXcodeArgs(&argc, argv);
@@ -348,6 +367,9 @@ parse_args(int argc, char *argv[], struct options *opts)
             case OPT_DEBUG:
                 opts->context_debug = true;
                 break;
+            case OPT_ROBUST:
+                opts->context_robust = true;
+                break;
             case OPT_RESIZE_WINDOW:
                 opts->resize_window = true;
                 break;
@@ -358,8 +380,12 @@ parse_args(int argc, char *argv[], struct options *opts)
                     usage_error_printf("'%s' is not a valid window geometry",
                                        optarg);
                 }
+                found_window_size = true;
                 break;
             }
+            case OPT_FULLSCREEN:
+                opts->fullscreen = true;
+                break;
             default:
                 abort();
                 loop_get_opt = false;
@@ -379,6 +405,11 @@ parse_args(int argc, char *argv[], struct options *opts)
         usage_error_printf("--api is required");
     }
 
+    if (opts->fullscreen && found_window_size) {
+        usage_error_printf("--fullscreen and --window-size are mutually "
+                           "exclusive options");
+    }
+
     // Set dl.
     switch (opts->context_api) {
         case WAFFLE_CONTEXT_OPENGL:     opts->dl = WAFFLE_DL_OPENGL;      break;
@@ -396,6 +427,29 @@ error_unrecognized_arg:
     usage_error_printf("unrecognized option '%s'", optarg);
 }
 
+// The rules that dictate how to properly query a GL symbol are complex. The
+// rules depend on the OS, on the winsys API, and even on the particular driver
+// being used. The rules differ between EGL 1.4 and EGL 1.5; differ between
+// Linux, Windows, and Mac; and differ between Mesa and Mali.
+//
+// This function hides that complexity with a naive heuristic: try, then try
+// again.
+static void *
+get_gl_symbol(const struct options *opts, const char *name)
+{
+    void *sym = NULL;
+
+    if (waffle_dl_can_open(opts->dl)) {
+        sym = waffle_dl_sym(opts->dl, name);
+    }
+
+    if (!sym) {
+        sym = waffle_get_proc_address(name);
+    }
+
+    return sym;
+}
+
 static bool
 draw(struct waffle_window *window, bool resize)
 {
@@ -520,8 +574,20 @@ removeXcodeArgs(int *argc, char **argv)
 
 #endif // __APPLE__
 
+#ifdef __native_client__
+#include "ppapi_simple/ps_main.h"
+//
+// We need to rename main() for native client
+// because ppapi_simple already defines main().
+//
+int basic_test_main(int argc, char **argv);
+PPAPI_SIMPLE_REGISTER_MAIN(basic_test_main)
+int
+basic_test_main(int argc, char **argv)
+#else
 int
 main(int argc, char **argv)
+#endif
 {
     bool ok;
     int i;
@@ -530,6 +596,7 @@ main(int argc, char **argv)
 
     int32_t init_attrib_list[3];
     int32_t config_attrib_list[64];
+    intptr_t window_attrib_list[5];
 
     struct waffle_display *dpy;
     struct waffle_config *config;
@@ -542,9 +609,18 @@ main(int argc, char **argv)
         cocoa_init();
     #endif
 
+    #ifdef __native_client__
+        // Fixed arguments for native client.
+        opts.context_api = WAFFLE_CONTEXT_OPENGL_ES2;
+        opts.platform = WAFFLE_PLATFORM_NACL;
+        opts.dl = WAFFLE_DL_OPENGL_ES2;
+        opts.context_profile = WAFFLE_NONE;
+        opts.context_version = -1;
+    #else
     ok = parse_args(argc, argv, &opts);
     if (!ok)
         exit(EXIT_FAILURE);
+    #endif
 
     i = 0;
     init_attrib_list[i++] = WAFFLE_PLATFORM;
@@ -564,23 +640,27 @@ main(int argc, char **argv)
                      waffle_enum_to_string(opts.context_api));
     }
 
-    glClear = waffle_dl_sym(opts.dl, "glClear");
+    glClear = get_gl_symbol(&opts, "glClear");
     if (!glClear)
         error_get_gl_symbol("glClear");
 
-    glClearColor = waffle_dl_sym(opts.dl, "glClearColor");
+    glClearColor = get_gl_symbol(&opts, "glClearColor");
     if (!glClearColor)
         error_get_gl_symbol("glClearColor");
 
-    glGetIntegerv = waffle_dl_sym(opts.dl, "glGetIntegerv");
+    glGetError = get_gl_symbol(&opts, "glGetError");
+    if (!glGetError)
+        error_get_gl_symbol("glGetError");
+
+    glGetIntegerv = get_gl_symbol(&opts, "glGetIntegerv");
     if (!glGetIntegerv)
         error_get_gl_symbol("glGetIntegerv");
 
-    glReadPixels = waffle_dl_sym(opts.dl, "glReadPixels");
+    glReadPixels = get_gl_symbol(&opts, "glReadPixels");
     if (!glReadPixels)
         error_get_gl_symbol("glReadPixels");
 
-    glViewport = waffle_dl_sym(opts.dl, "glViewport");
+    glViewport = get_gl_symbol(&opts, "glViewport");
     if (!glViewport)
         error_get_gl_symbol("glViewport");
 
@@ -610,12 +690,19 @@ main(int argc, char **argv)
         config_attrib_list[i++] = true;
     }
 
+    if (opts.context_robust) {
+        config_attrib_list[i++] = WAFFLE_CONTEXT_ROBUST_ACCESS,
+        config_attrib_list[i++] = true;
+    }
+
     config_attrib_list[i++] = WAFFLE_RED_SIZE;
     config_attrib_list[i++] = 8;
     config_attrib_list[i++] = WAFFLE_GREEN_SIZE;
     config_attrib_list[i++] = 8;
     config_attrib_list[i++] = WAFFLE_BLUE_SIZE;
     config_attrib_list[i++] = 8;
+    config_attrib_list[i++] = WAFFLE_ALPHA_SIZE;
+    config_attrib_list[i++] = 8;
     config_attrib_list[i++] = WAFFLE_DOUBLE_BUFFERED;
     config_attrib_list[i++] = true;
     config_attrib_list[i++] = 0;
@@ -628,7 +715,21 @@ main(int argc, char **argv)
     if (!ctx)
         error_waffle();
 
-    window = waffle_window_create(config, window_width, window_height);
+
+    i = 0;
+    if (opts.fullscreen) {
+        window_attrib_list[i++] = WAFFLE_WINDOW_FULLSCREEN;
+        window_attrib_list[i++] = true;
+        window_attrib_list[i++] = 0;
+    } else {
+        window_attrib_list[i++] = WAFFLE_WINDOW_WIDTH;
+        window_attrib_list[i++] = window_width;
+        window_attrib_list[i++] = WAFFLE_WINDOW_HEIGHT;
+        window_attrib_list[i++] = window_height;
+        window_attrib_list[i++] = 0;
+    }
+
+    window = waffle_window_create2(config, window_attrib_list);
     if (!window)
         error_waffle();
 
@@ -640,6 +741,9 @@ main(int argc, char **argv)
         glGetIntegerv(GL_CONTEXT_FLAGS, &context_flags);
     }
 
+    if (glGetError())
+        error_printf("glGetIntegerv(GL_CONTEXT_FLAGS) failed");
+
     if (opts.context_forward_compatible
         && !(context_flags & GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT)) {
         error_printf("context is not forward-compatible");
@@ -650,6 +754,11 @@ main(int argc, char **argv)
         error_printf("context is not a debug context");
     }
 
+    // Don't verify if the context is robust because that pollutes this simple
+    // example program with hairy GL logic. The method of verifying if
+    // a context is robust varies on the combination of context profile,
+    // context version, and supported extensions.
+
     ok = draw(window, opts.resize_window);
     if (!ok)
         error_waffle();
@@ -674,6 +783,10 @@ main(int argc, char **argv)
     if (!ok)
         error_waffle();
 
+    ok = waffle_teardown();
+    if (!ok)
+        error_waffle();
+
     #ifdef __APPLE__
         cocoa_finish();
     #endif
