@@ -35,6 +35,8 @@ struct wcore_platform* wayland_platform_create(void);
 struct wcore_platform* xegl_platform_create(void);
 struct wcore_platform* wgbm_platform_create(void);
 struct wcore_platform* wgl_platform_create(void);
+struct wcore_platform* nacl_platform_create(void);
+struct wcore_platform* sl_platform_create(void);
 
 static bool
 waffle_init_parse_attrib_list(
@@ -104,6 +106,18 @@ waffle_init_parse_attrib_list(
                     CASE_UNDEFINED_PLATFORM(WGL)
 #endif
 
+#ifdef WAFFLE_HAS_NACL
+                    CASE_DEFINED_PLATFORM(NACL)
+#else
+                    CASE_UNDEFINED_PLATFORM(NACL)
+#endif
+
+#ifdef WAFFLE_HAS_SURFACELESS_EGL
+                    CASE_DEFINED_PLATFORM(SURFACELESS_EGL)
+#else
+                    CASE_UNDEFINED_PLATFORM(SURFACELESS_EGL)
+#endif
+
                     default:
                         wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
                                      "WAFFLE_PLATFORM has bad value 0x%x",
@@ -135,39 +149,63 @@ waffle_init_parse_attrib_list(
 static struct wcore_platform*
 waffle_init_create_platform(int32_t waffle_platform)
 {
+    struct wcore_platform *wc_platform = NULL;
+
     switch (waffle_platform) {
 #ifdef WAFFLE_HAS_ANDROID
         case WAFFLE_PLATFORM_ANDROID:
-            return droid_platform_create();
+            wc_platform = droid_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_CGL
         case WAFFLE_PLATFORM_CGL:
-            return cgl_platform_create();
+            wc_platform = cgl_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_GLX
         case WAFFLE_PLATFORM_GLX:
-            return glx_platform_create();
+            wc_platform = glx_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_WAYLAND
         case  WAFFLE_PLATFORM_WAYLAND:
-            return wayland_platform_create();
+            wc_platform = wayland_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_X11_EGL
         case WAFFLE_PLATFORM_X11_EGL:
-            return xegl_platform_create();
+            wc_platform = xegl_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_GBM
         case WAFFLE_PLATFORM_GBM:
-            return wgbm_platform_create();
+            wc_platform = wgbm_platform_create();
+            break;
 #endif
 #ifdef WAFFLE_HAS_WGL
         case WAFFLE_PLATFORM_WGL:
-            return wgl_platform_create();
+            wc_platform = wgl_platform_create();
+            break;
+#endif
+#ifdef WAFFLE_HAS_NACL
+        case WAFFLE_PLATFORM_NACL:
+            wc_platform = nacl_platform_create();
+            break;
+#endif
+#ifdef WAFFLE_HAS_SURFACELESS_EGL
+        case WAFFLE_PLATFORM_SURFACELESS_EGL:
+            wc_platform = sl_platform_create();
+            break;
 #endif
         default:
             assert(false);
             return NULL;
     }
+
+    if (wc_platform)
+        wc_platform->waffle_platform = waffle_platform;
+
+    return wc_platform;
 }
 
 WAFFLE_API bool
@@ -193,3 +231,23 @@ waffle_init(const int32_t *attrib_list)
 
     return true;
 }
+
+WAFFLE_API bool
+waffle_teardown(void)
+{
+    bool ok = true;
+
+    wcore_error_reset();
+
+    if (!api_platform) {
+        wcore_error(WAFFLE_ERROR_NOT_INITIALIZED);
+        return false;
+    }
+
+    ok &= api_platform->vtbl->destroy(api_platform);
+    if (!ok)
+        return false;
+
+    api_platform = NULL;
+    return true;
+}
