@@ -54,6 +54,7 @@ check_keys(const int32_t attrib_list[])
             case WAFFLE_CONTEXT_PROFILE:
             case WAFFLE_CONTEXT_FORWARD_COMPATIBLE:
             case WAFFLE_CONTEXT_DEBUG:
+            case WAFFLE_CONTEXT_ROBUST_ACCESS:
             case WAFFLE_RED_SIZE:
             case WAFFLE_GREEN_SIZE:
             case WAFFLE_BLUE_SIZE:
@@ -77,27 +78,25 @@ check_keys(const int32_t attrib_list[])
 }
 
 static bool
-parse_bool(const int32_t attrib_list[],
-		   int32_t attrib_name,
-		   bool *value,
-		   bool default_value)
+parse_bool(const int32_t attrib_list[], int32_t attrib_name,
+           bool *value, bool default_value)
 {
-	int32_t raw_value;
+    int32_t raw_value;
 
-	wcore_attrib_list_get_with_default(attrib_list, attrib_name,
+    wcore_attrib_list32_get_with_default(attrib_list, attrib_name,
                                        &raw_value, default_value);
 
-	if (raw_value == WAFFLE_DONT_CARE) {
-		*value = default_value;
-	} else if (raw_value == true || raw_value == false) {
-		*value = raw_value;
-	} else {
-		wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
-					 "%s has bad value 0x%x. "
-					 "Must be true(1), false(0), or WAFFLE_DONT_CARE(-1)",
-					 wcore_enum_to_string(attrib_name), raw_value);
-		return false;
-	}
+    if (raw_value == WAFFLE_DONT_CARE) {
+        *value = default_value;
+    } else if (raw_value == true || raw_value == false) {
+        *value = raw_value;
+    } else {
+        wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
+                    "%s has bad value 0x%x. "
+                    "Must be true(1), false(0), or WAFFLE_DONT_CARE(-1)",
+                    wcore_enum_to_string(attrib_name), raw_value);
+        return false;
+    }
 
     return true;
 }
@@ -108,7 +107,7 @@ parse_context_api(struct wcore_config_attrs *attrs,
 {
     bool found;
 
-    found = wcore_attrib_list_get(attrib_list,
+    found = wcore_attrib_list32_get(attrib_list,
                                   WAFFLE_CONTEXT_API, &attrs->context_api);
     if (!found) {
         wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
@@ -153,8 +152,7 @@ set_context_version_default(struct wcore_config_attrs *attrs)
             attrs->context_minor_version = 0;
             return true;
         default:
-            wcore_error_internal("attrs->context_api has bad value 0x%x",
-                                 attrs->context_api);
+            assert(false);
             return false;
     }
 }
@@ -163,14 +161,10 @@ static bool
 parse_context_version(struct wcore_config_attrs *attrs,
                       const int32_t attrib_list[])
 {
-    wcore_attrib_list_get_with_default(attrib_list,
-                                       WAFFLE_CONTEXT_MAJOR_VERSION,
-                                       &attrs->context_major_version,
-                                       attrs->context_major_version);
-    wcore_attrib_list_get_with_default(attrib_list,
-                                       WAFFLE_CONTEXT_MINOR_VERSION,
-                                       &attrs->context_minor_version,
-                                       attrs->context_minor_version);
+    wcore_attrib_list32_get(attrib_list, WAFFLE_CONTEXT_MAJOR_VERSION,
+                            &attrs->context_major_version);
+    wcore_attrib_list32_get(attrib_list, WAFFLE_CONTEXT_MINOR_VERSION,
+                            &attrs->context_minor_version);
 
     if (attrs->context_major_version < 1) {
         wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
@@ -183,51 +177,6 @@ parse_context_version(struct wcore_config_attrs *attrs,
                      "WAFFLE_CONTEXT_MINOR_VERSION must be >= 0");
         return false;
     }
-
-    switch (attrs->context_api) {
-        case WAFFLE_CONTEXT_OPENGL:
-            if (wcore_config_attrs_version_lt(attrs, 10)) {
-                wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
-                             "for OpenGL, the requested context version "
-                             "must be >= 1.0");
-                return false;
-            }
-            break;
-
-        case WAFFLE_CONTEXT_OPENGL_ES1:
-            if (!wcore_config_attrs_version_eq(attrs, 10) &&
-                !wcore_config_attrs_version_eq(attrs, 11)) {
-                wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
-                             "for OpenGL ES1, the requested context version "
-                             "must be 1.0 or 1.1");
-                return false;
-            }
-            break;
-
-        case WAFFLE_CONTEXT_OPENGL_ES2:
-            if (attrs->context_major_version != 2) {
-                wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
-                             "for OpenGL ES2, the requested major context "
-                             "version must be 2");
-                return false;
-            }
-            break;
-
-        case WAFFLE_CONTEXT_OPENGL_ES3:
-            if (attrs->context_major_version != 3) {
-                wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
-                             "for OpenGL ES3, the requested major context "
-                             "version must be 3");
-                return false;
-            }
-            break;
-
-        default:
-            wcore_error_internal("attrs->context_api has bad value 0x%x",
-                                 attrs->context_api);
-            return false;
-    }
-
     return true;
 }
 
@@ -260,10 +209,8 @@ static bool
 parse_context_profile(struct wcore_config_attrs *attrs,
                       const int32_t attrib_list[])
 {
-    wcore_attrib_list_get_with_default(attrib_list,
-                                       WAFFLE_CONTEXT_PROFILE,
-                                       &attrs->context_profile,
-                                       attrs->context_profile);
+    wcore_attrib_list32_get(attrib_list, WAFFLE_CONTEXT_PROFILE,
+                            &attrs->context_profile);
 
     switch (attrs->context_api) {
         case WAFFLE_CONTEXT_OPENGL:
@@ -345,6 +292,7 @@ set_misc_defaults(struct wcore_config_attrs *attrs)
     // [2] EGL 1.4 spec (2011.04.06), Table 3.4
 
     attrs->context_debug        = false;
+    attrs->context_robust       = false;
 
     attrs->rgba_size            = 0;
     attrs->red_size             = 0;
@@ -420,6 +368,7 @@ parse_misc(struct wcore_config_attrs *attrs,
             CASE_INT(WAFFLE_SAMPLES, samples)
 
             CASE_BOOL(WAFFLE_CONTEXT_DEBUG, context_debug, false);
+            CASE_BOOL(WAFFLE_CONTEXT_ROBUST_ACCESS, context_robust, false);
             CASE_BOOL(WAFFLE_SAMPLE_BUFFERS, sample_buffers, DEFAULT_SAMPLE_BUFFERS);
             CASE_BOOL(WAFFLE_DOUBLE_BUFFERED, double_buffered, DEFAULT_DOUBLE_BUFFERED);
             CASE_BOOL(WAFFLE_ACCUM_BUFFER, accum_buffer, DEFAULT_ACCUM_BUFFER);
