@@ -25,12 +25,18 @@
 
 #pragma once
 
+#include <assert.h>
 #include "c99_compat.h"
+#include "threads.h"
 
 #include "api_object.h"
 
 #include "wcore_util.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct wcore_display;
 struct wcore_platform;
 union waffle_native_display;
@@ -55,5 +61,14 @@ wcore_display_init(struct wcore_display *self,
                    struct wcore_platform *platform);
 
 
-bool
-wcore_display_teardown(struct wcore_display *self);
+static inline bool
+wcore_display_teardown(struct wcore_display *self)
+{
+    (void) self;
+    assert(self);
+    return true;
+}
+
+#ifdef __cplusplus
+}
+#endif
