@@ -24,6 +24,7 @@
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <errno.h>
+#include <inttypes.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -169,6 +170,13 @@ wcore_error_errno(const char *format, ...)
    strerror_r(saved_errno, cur, end - cur);
 }
 
+void
+wcore_error_bad_attribute(intptr_t attr)
+{
+    wcore_errorf(WAFFLE_ERROR_BAD_ATTRIBUTE,
+                 "bad attribute 0x%"PRIxPTR, attr);
+}
+
 void
 _wcore_error_internal(const char *file, int line, const char *format, ...)
 {
@@ -204,7 +212,7 @@ _wcore_error_internal(const char *file, int line, const char *format, ...)
             return;
     }
 
-    snprintf(cur, end - cur, " ; Please report bug at https://github.com/waffle-gl/waffle/issues");
+    snprintf(cur, end - cur, " ; Please report bug at https://gitlab.freedesktop.org/mesa/waffle/issues");
 }
 
 enum waffle_error
