@@ -34,13 +34,21 @@
 #include "wegl_platform.h"
 
 static bool
-get_extensions(struct wegl_display *dpy)
+get_apis(struct wegl_display *dpy)
 {
     struct wegl_platform *plat = wegl_platform(dpy->wcore.platform);
-    const char *extensions = plat->eglQueryString(dpy->egl, EGL_EXTENSIONS);
+    const char *apis = plat->eglQueryString(dpy->egl, EGL_CLIENT_APIS);
 
-    if (!extensions) {
-        wegl_emit_error(plat, "eglQueryString(EGL_EXTENSIONS");
+    // Our minimum requirement - EGL 1.2 ...
+    if (dpy->major_version != 1 || dpy->minor_version < 2) {
+        wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
+                     "EGL 1.2 or later is required");
+        return false;
+    }
+
+    // ... plus working eglQueryString(EGL_CLIENT_APIS).
+    if (!apis) {
+        wegl_emit_error(plat, "eglQueryString(EGL_CLIENT_APIS)");
         return false;
     }
 
@@ -49,7 +57,41 @@ get_extensions(struct wegl_display *dpy)
     // pending emission.
     assert(wcore_error_get_code() == 0);
 
-    dpy->KHR_create_context = waffle_is_extension_in_string(extensions, "EGL_KHR_create_context");
+    if (waffle_is_extension_in_string(apis, "OpenGL_ES"))
+        dpy->api_mask |= WEGL_OPENGL_ES_API;
+
+    // Check for "OpenGL" if we're running EGL 1.4 or later.
+    if (dpy->major_version == 1 && dpy->minor_version >= 4)
+        if (waffle_is_extension_in_string(apis, "OpenGL"))
+            dpy->api_mask |= WEGL_OPENGL_API;
+
+    return true;
+}
+
+static bool
+get_extensions(struct wegl_display *dpy)
+{
+    struct wegl_platform *plat = wegl_platform(dpy->wcore.platform);
+    const char *extensions = plat->eglQueryString(dpy->egl, EGL_EXTENSIONS);
+
+    if (!extensions) {
+        wegl_emit_error(plat, "eglQueryString(EGL_EXTENSIONS)");
+        return false;
+    }
+
+    // waffle_is_extension_in_string() resets the error state. That's ok,
+    // however, because if we've reached this point then no error should be
+    // pending emission.
+    assert(wcore_error_get_code() == 0);
+
+#define CHECK_EXTENSION(ext) \
+    dpy->ext = waffle_is_extension_in_string(extensions, "EGL_" #ext)
+
+    CHECK_EXTENSION(EXT_create_context_robustness);
+    CHECK_EXTENSION(KHR_create_context);
+    CHECK_EXTENSION(EXT_image_dma_buf_import_modifiers);
+
+#undef CHECK_EXTENSION
 
     return true;
 }
@@ -59,28 +101,47 @@ get_extensions(struct wegl_display *dpy)
 bool
 wegl_display_init(struct wegl_display *dpy,
                   struct wcore_platform *wc_plat,
-                  intptr_t native_display)
+                  void *native_display)
 {
     struct wegl_platform *plat = wegl_platform(wc_plat);
     bool ok;
-    EGLint major, minor;
 
     ok = wcore_display_init(&dpy->wcore, wc_plat);
     if (!ok)
         goto fail;
 
-    dpy->egl = plat->eglGetDisplay((EGLNativeDisplayType) native_display);
-    if (!dpy->egl) {
-        wegl_emit_error(plat, "eglGetDisplay");
-        goto fail;
+    if (wegl_platform_can_use_eglGetPlatformDisplay(plat)) {
+        dpy->egl = plat->eglGetPlatformDisplay(plat->egl_platform,
+                                               native_display, NULL);
+        if (!dpy->egl) {
+            wegl_emit_error(plat, "eglGetPlatformDisplay");
+            goto fail;
+        }
+    } else if (wegl_platform_can_use_eglGetPlatformDisplayEXT(plat)) {
+        dpy->egl = plat->eglGetPlatformDisplayEXT(plat->egl_platform,
+                                                  native_display, NULL);
+        if (!dpy->egl) {
+            wegl_emit_error(plat, "eglGetPlatformDisplayEXT");
+            goto fail;
+        }
+    } else {
+        dpy->egl = plat->eglGetDisplay((EGLNativeDisplayType) native_display);
+        if (!dpy->egl) {
+            wegl_emit_error(plat, "eglGetDisplay");
+            goto fail;
+        }
     }
 
-    ok = plat->eglInitialize(dpy->egl, &major, &minor);
+    ok = plat->eglInitialize(dpy->egl, &dpy->major_version, &dpy->minor_version);
     if (!ok) {
         wegl_emit_error(plat, "eglInitialize");
         goto fail;
     }
 
+    ok = get_apis(dpy);
+    if (!ok)
+        goto fail;
+
     ok = get_extensions(dpy);
     if (!ok)
         goto fail;
@@ -112,30 +173,18 @@ wegl_display_supports_context_api(struct wcore_display *wc_dpy,
                                   int32_t waffle_context_api)
 {
     struct wegl_display *dpy = wegl_display(wc_dpy);
-    struct wcore_platform *wc_plat = dpy->wcore.platform;
-    int32_t waffle_dl;
 
     switch (waffle_context_api) {
         case WAFFLE_CONTEXT_OPENGL:
-            waffle_dl = WAFFLE_DL_OPENGL;
-            break;
+            return !!(dpy->api_mask & WEGL_OPENGL_API);
         case WAFFLE_CONTEXT_OPENGL_ES1:
-            waffle_dl = WAFFLE_DL_OPENGL_ES1;
-            break;
         case WAFFLE_CONTEXT_OPENGL_ES2:
-            waffle_dl = WAFFLE_DL_OPENGL_ES2;
-            break;
+            return !!(dpy->api_mask & WEGL_OPENGL_ES_API);
         case WAFFLE_CONTEXT_OPENGL_ES3:
-            if (!dpy->KHR_create_context)
-                return false;
-
-            waffle_dl = WAFFLE_DL_OPENGL_ES3;
-            break;
+            return !!(dpy->api_mask & WEGL_OPENGL_ES_API) &&
+                   dpy->KHR_create_context;
         default:
-            wcore_error_internal("waffle_context_api has bad value %#x",
-                                 waffle_context_api);
+            assert(false);
             return false;
     }
-
-    return wc_plat->vtbl->dl_can_open(wc_plat, waffle_dl);
 }
