@@ -23,8 +23,6 @@
 // OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#define _POSIX_C_SOURCE 200112 // glib feature macro for unsetenv()
-
 #include <stdlib.h>
 #include <dlfcn.h>
 
@@ -46,18 +44,15 @@ static const char *libgbm_filename = "libgbm.so.1";
 
 static const struct wcore_platform_vtbl wgbm_platform_vtbl;
 
-static bool
-wgbm_platform_destroy(struct wcore_platform *wc_self)
+bool
+wgbm_platform_teardown(struct wgbm_platform *self)
 {
-    struct wgbm_platform *self = wgbm_platform(wegl_platform(wc_self));
     bool ok = true;
     int error = 0;
 
     if (!self)
         return true;
 
-    unsetenv("EGL_PLATFORM");
-
     if (self->linux)
         ok &= linux_platform_destroy(self->linux);
 
@@ -72,21 +67,28 @@ wgbm_platform_destroy(struct wcore_platform *wc_self)
     }
 
     ok &= wegl_platform_teardown(&self->wegl);
+    return ok;
+}
+
+bool
+wgbm_platform_destroy(struct wcore_platform *wc_self)
+{
+    struct wgbm_platform *self = wgbm_platform(wegl_platform(wc_self));
+
+    if (!self)
+        return true;
+
+    bool ok = wgbm_platform_teardown(self);
     free(self);
     return ok;
 }
 
-struct wcore_platform*
-wgbm_platform_create(void)
+bool
+wgbm_platform_init(struct wgbm_platform *self)
 {
-    struct wgbm_platform *self;
     bool ok = true;
 
-    self = wcore_calloc(sizeof(*self));
-    if (self == NULL)
-        return NULL;
-
-    ok = wegl_platform_init(&self->wegl);
+    ok = wegl_platform_init(&self->wegl, EGL_PLATFORM_GBM_KHR);
     if (!ok)
         goto error;
 
@@ -98,41 +100,46 @@ wgbm_platform_create(void)
         goto error;
     }
 
-#define RETRIEVE_GBM_SYMBOL(function)                                  \
+#define RETRIEVE_GBM_SYMBOL(type, function, required, args)            \
     self->function = dlsym(self->gbmHandle, #function);                \
-    if (!self->function) {                                             \
+    if (required && !self->function) {                                 \
         wcore_errorf(WAFFLE_ERROR_FATAL,                             \
                      "dlsym(\"%s\", \"" #function "\") failed: %s",    \
                      libgbm_filename, dlerror());                      \
         goto error;                                                    \
     }
 
-    RETRIEVE_GBM_SYMBOL(gbm_create_device);
-    RETRIEVE_GBM_SYMBOL(gbm_device_get_fd);
-    RETRIEVE_GBM_SYMBOL(gbm_device_destroy);
-
-    RETRIEVE_GBM_SYMBOL(gbm_surface_create);
-    RETRIEVE_GBM_SYMBOL(gbm_surface_destroy);
-
-    RETRIEVE_GBM_SYMBOL(gbm_surface_lock_front_buffer);
-    RETRIEVE_GBM_SYMBOL(gbm_surface_release_buffer);
+    GBM_FUNCTIONS(RETRIEVE_GBM_SYMBOL);
 #undef RETRIEVE_GBM_SYMBOL
 
     self->linux = linux_platform_create();
     if (!self->linux)
         goto error;
 
-    setenv("EGL_PLATFORM", "drm", true);
-
     self->wegl.wcore.vtbl = &wgbm_platform_vtbl;
-    return &self->wegl.wcore;
+    return true;
 
 error:
-    wgbm_platform_destroy(&self->wegl.wcore);
-    return NULL;
+    wgbm_platform_teardown(self);
+    return false;
 }
 
-static bool
+struct wcore_platform*
+wgbm_platform_create(void)
+{
+    struct wgbm_platform *self = wcore_calloc(sizeof(*self));
+    if (self == NULL)
+        return NULL;
+
+    if (!wgbm_platform_init(self)) {
+        wgbm_platform_destroy(&self->wegl.wcore);
+        return NULL;
+    }
+
+    return &self->wegl.wcore;
+}
+
+bool
 wgbm_dl_can_open(struct wcore_platform *wc_self,
                  int32_t waffle_dl)
 {
@@ -140,7 +147,7 @@ wgbm_dl_can_open(struct wcore_platform *wc_self,
     return linux_platform_dl_can_open(self->linux, waffle_dl);
 }
 
-static void*
+void*
 wgbm_dl_sym(struct wcore_platform *wc_self,
             int32_t waffle_dl,
             const char *name)
@@ -182,7 +189,7 @@ static const struct wcore_platform_vtbl wgbm_platform_vtbl = {
     },
 
     .config = {
-        .choose = wgbm_config_choose,
+        .choose = wegl_config_choose,
         .destroy = wegl_config_destroy,
         .get_native = wgbm_config_get_native,
     },
@@ -198,6 +205,7 @@ static const struct wcore_platform_vtbl wgbm_platform_vtbl = {
         .destroy = wgbm_window_destroy,
         .show = wgbm_window_show,
         .swap_buffers = wgbm_window_swap_buffers,
+        .resize = wgbm_window_resize,
         .get_native = wgbm_window_get_native,
     },
 };
