@@ -55,11 +55,6 @@ static bool
 glx_config_check_context_attrs(struct glx_display *dpy,
                                const struct wcore_config_attrs *attrs)
 {
-    if (attrs->context_forward_compatible) {
-        assert(attrs->context_api == WAFFLE_CONTEXT_OPENGL);
-        assert(wcore_config_attrs_version_ge(attrs, 30));
-    }
-
     if (!dpy->ARB_create_context &&
         glx_context_needs_arb_create_context(attrs)) {
         const char *gl = "";
@@ -103,40 +98,18 @@ glx_config_check_context_attrs(struct glx_display *dpy,
                              "to create a context with version >= 3.2");
                 return false;
             }
-            else if (wcore_config_attrs_version_ge(attrs, 32)) {
-                assert(attrs->context_profile == WAFFLE_CONTEXT_CORE_PROFILE ||
-                       attrs->context_profile == WAFFLE_CONTEXT_COMPATIBILITY_PROFILE);
-            }
 
             return true;
 
         case WAFFLE_CONTEXT_OPENGL_ES1:
-            if (!dpy->EXT_create_context_es_profile) {
-                wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
-                             "GLX_EXT_create_context_es_profile is required "
-                             "to create an OpenGL ES1 context");
-                return false;
-            }
-
-            return true;
-
         case WAFFLE_CONTEXT_OPENGL_ES2:
-            if (!dpy->EXT_create_context_es_profile
-                && !dpy->EXT_create_context_es2_profile) {
+        case WAFFLE_CONTEXT_OPENGL_ES3:
+            if (!dpy->EXT_create_context_es_profile &&
+                !dpy->EXT_create_context_es2_profile) {
                 wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
                              "GLX_EXT_create_context_es_profile or "
                              "GLX_EXT_create_context_es2_profile is required "
-                             "to create an OpenGL ES2 context");
-                return false;
-            }
-
-            return true;
-
-        case WAFFLE_CONTEXT_OPENGL_ES3:
-            if (!dpy->EXT_create_context_es_profile) {
-                wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
-                             "GLX_EXT_create_context_es_profile is required "
-                             "to create an OpenGL ES3 context");
+                             "to create an OpenGL ES* context");
                 return false;
             }
 
