@@ -1,10 +1,3 @@
-WAFFLE_WAYLAND_INTERFACE(wl_compositor_interface)
-WAFFLE_WAYLAND_INTERFACE(wl_registry_interface)
-WAFFLE_WAYLAND_INTERFACE(wl_shell_interface)
-WAFFLE_WAYLAND_INTERFACE(wl_shell_surface_interface)
-WAFFLE_WAYLAND_INTERFACE(wl_surface_interface)
-
-
 WAFFLE_WAYLAND_SYM(struct wl_display *, wl_display_connect, (const char *name))
 WAFFLE_WAYLAND_SYM(void, wl_display_disconnect, (struct wl_display *display))
 
@@ -16,6 +9,13 @@ WAFFLE_WAYLAND_SYM(int, wl_proxy_add_listener, (struct wl_proxy *proxy,
                                                 void (**implementation)(void),
                                                 void *data))
 
+WAFFLE_WAYLAND_SYM(void, wl_proxy_set_user_data, (struct wl_proxy *proxy,
+                                                  void *user_data))
+
+WAFFLE_WAYLAND_SYM(void *, wl_proxy_get_user_data, (struct wl_proxy *proxy))
+
+WAFFLE_WAYLAND_SYM(uint32_t, wl_proxy_get_version, (struct wl_proxy *proxy))
+
 WAFFLE_WAYLAND_SYM(void, wl_proxy_marshal, (struct wl_proxy *p,
                                             uint32_t opcode, ...))
 
