@@ -43,6 +43,7 @@
 #include "wayland_display.h"
 #include "wayland_platform.h"
 #include "wayland_window.h"
+#include "wl-xdg-shell-proto.h"
 
 bool
 wayland_window_destroy(struct wcore_window *wc_self)
@@ -63,6 +64,12 @@ wayland_window_destroy(struct wcore_window *wc_self)
     if (self->wl_shell_surface)
         wl_shell_surface_destroy(self->wl_shell_surface);
 
+    if (self->xdg_toplevel)
+        xdg_toplevel_destroy(self->xdg_toplevel);
+
+    if (self->xdg_surface)
+        xdg_surface_destroy(self->xdg_surface);
+
     if (self->wl_surface)
         wl_surface_destroy(self->wl_surface);
 
@@ -70,6 +77,63 @@ wayland_window_destroy(struct wcore_window *wc_self)
     return ok;
 }
 
+static void
+surface_handle_configure(void *data, struct xdg_surface *surface,
+                         uint32_t serial)
+{
+    xdg_surface_ack_configure(surface, serial);
+}
+
+static const struct xdg_surface_listener surface_listener = {
+    .configure = surface_handle_configure,
+};
+
+static void
+toplevel_handle_configure(void *data, struct xdg_toplevel *toplevel,
+                          int32_t width, int32_t height,
+                          struct wl_array *states)
+{
+    struct wcore_window *wc_self = data;
+    struct wayland_window *self = wayland_window(wc_self);
+    enum xdg_toplevel_state *state;
+    bool fullscreen = false, maximized = false;
+
+    wl_array_for_each(state, states) {
+        switch (*state) {
+        case XDG_TOPLEVEL_STATE_MAXIMIZED:
+            maximized = true;
+            break;
+        case XDG_TOPLEVEL_STATE_FULLSCREEN:
+            fullscreen = true;
+            break;
+        default:
+            break;
+        }
+    }
+
+    if (width > 0 && height > 0) {
+        if (!fullscreen && !maximized) {
+            self->window_width = width;
+            self->window_height = height;
+        }
+    } else if (!fullscreen && !maximized) {
+        width = self->window_width;
+        height = self->window_height;
+    }
+
+    wayland_window_resize(wc_self, width, height);
+}
+
+static void
+toplevel_handle_close(void *data, struct xdg_toplevel *xdg_toplevel)
+{
+}
+
+static const struct xdg_toplevel_listener toplevel_listener = {
+    .configure = toplevel_handle_configure,
+    .close = toplevel_handle_close,
+};
+
 static void
 shell_surface_listener_ping(void *data,
                             struct wl_shell_surface *shell_surface,
@@ -109,9 +173,13 @@ wayland_window_create(struct wcore_platform *wc_plat,
     struct wayland_window *self;
     struct wayland_platform *plat = wayland_platform(wegl_platform(wc_plat));
     struct wayland_display *dpy = wayland_display(wc_config->display);
+    bool fullscreen = false;
     bool ok = true;
 
-    if (width == -1 && height == -1) {
+    if (width == -1 && height == -1)
+        fullscreen = true;
+
+    if (dpy->wl_shell && fullscreen) {
         wcore_errorf(WAFFLE_ERROR_UNSUPPORTED_ON_PLATFORM,
                      "fullscreen window not supported");
         return NULL;
@@ -130,7 +198,7 @@ wayland_window_create(struct wcore_platform *wc_plat,
         wcore_errorf(WAFFLE_ERROR_UNKNOWN, "wayland compositor not found");
         goto error;
     }
-    if (!dpy->wl_shell) {
+    if (!dpy->xdg_shell && !dpy->wl_shell) {
         wcore_errorf(WAFFLE_ERROR_UNKNOWN, "wayland shell not found");
         goto error;
     }
@@ -142,17 +210,46 @@ wayland_window_create(struct wcore_platform *wc_plat,
         goto error;
     }
 
-    self->wl_shell_surface = wl_shell_get_shell_surface(dpy->wl_shell,
+    if (dpy->xdg_shell) {
+        self->xdg_surface = xdg_wm_base_get_xdg_surface(dpy->xdg_shell,
                                                         self->wl_surface);
-    if (!self->wl_shell_surface) {
-        wcore_errorf(WAFFLE_ERROR_UNKNOWN,
-                     "wl_shell_get_shell_surface failed");
-        goto error;
-    }
+        if (!self->xdg_surface) {
+            wcore_errorf(WAFFLE_ERROR_UNKNOWN,
+                         "xdg_wm_base_get_xdg_surface failed");
+            goto error;
+        }
 
-    wl_shell_surface_add_listener(self->wl_shell_surface,
-                                  &shell_surface_listener,
-                                  NULL);
+        xdg_surface_add_listener(self->xdg_surface, &surface_listener, self);
+
+        self->xdg_toplevel = xdg_surface_get_toplevel(self->xdg_surface);
+        if (!self->xdg_toplevel) {
+            wcore_errorf(WAFFLE_ERROR_UNKNOWN,
+                         "xdg_surface_get_toplevel failed");
+            goto error;
+        }
+
+        xdg_toplevel_add_listener(self->xdg_toplevel, &toplevel_listener,
+                                 &self->wegl.wcore);
+    }
+    else {
+        self->wl_shell_surface = wl_shell_get_shell_surface(dpy->wl_shell,
+                                                            self->wl_surface);
+
+        if (!self->wl_shell_surface) {
+            wcore_errorf(WAFFLE_ERROR_UNKNOWN,
+                         "wl_shell_get_shell_surface failed");
+            goto error;
+        }
+
+        wl_shell_surface_add_listener(self->wl_shell_surface,
+                                      &shell_surface_listener,
+                                      NULL);
+
+        width = (width == -1 ? 320 : width);
+        height = (height == -1 ? 240 : height);
+        self->window_width = width;
+        self->window_height = height;
+    }
 
     self->wl_window = plat->wl_egl_window_create(self->wl_surface,
                                                  width, height);
@@ -161,6 +258,9 @@ wayland_window_create(struct wcore_platform *wc_plat,
         goto error;
     }
 
+    if (dpy->xdg_shell && fullscreen)
+        xdg_toplevel_set_fullscreen(self->xdg_toplevel, NULL);
+
     ok = wegl_window_init(&self->wegl, wc_config, (intptr_t) self->wl_window);
     if (!ok)
         goto error;
@@ -184,7 +284,10 @@ wayland_window_show(struct wcore_window *wc_self)
     struct wayland_display *dpy = wayland_display(wc_self->display);
     bool ok = true;
 
-    wl_shell_surface_set_toplevel(self->wl_shell_surface);
+    if (dpy->xdg_shell)
+        wl_surface_commit(self->wl_surface);
+    else
+        wl_shell_surface_set_toplevel(self->wl_shell_surface);
 
     ok = wayland_display_sync(dpy);
     if (!ok)
@@ -243,6 +346,8 @@ wayland_window_get_native(struct wcore_window *wc_self)
 
     wayland_display_fill_native(dpy, &n_window->wayland->display);
     n_window->wayland->wl_surface = self->wl_surface;
+    n_window->wayland->xdg_surface = self->xdg_surface;
+    n_window->wayland->xdg_toplevel = self->xdg_toplevel;
     n_window->wayland->wl_shell_surface = self->wl_shell_surface;
     n_window->wayland->wl_window = self->wl_window;
     n_window->wayland->egl_surface = self->wegl.egl;
