@@ -41,17 +41,12 @@ struct wl_proxy;
 struct wl_display;
 
 
-// Data symbols
-#define WAFFLE_WAYLAND_INTERFACE(iface) \
-    extern const struct wl_interface *wfl_##iface;
-
 // Functions
 #define WAFFLE_WAYLAND_SYM(rc, fn, params) \
     typedef rc (*pfn_##fn) params; \
     extern pfn_##fn wfl_##fn;
 #include "wayland_sym.h"
 #undef WAFFLE_WAYLAND_SYM
-#undef WAFFLE_WAYLAND_INTERFACE
 
 #ifdef _WAYLAND_CLIENT_H
 #error Do not include wayland-client.h ahead of wayland_wrapper.h
@@ -59,17 +54,14 @@ struct wl_display;
 
 #include <wayland-client-core.h>
 
-#define wl_compositor_interface (*wfl_wl_compositor_interface)
-#define wl_registry_interface (*wfl_wl_registry_interface)
-#define wl_shell_interface (*wfl_wl_shell_interface)
-#define wl_shell_surface_interface (*wfl_wl_shell_surface_interface)
-#define wl_surface_interface (*wfl_wl_surface_interface)
-
 #define wl_display_connect (*wfl_wl_display_connect)
 #define wl_display_disconnect (*wfl_wl_display_disconnect)
 #define wl_display_roundtrip (*wfl_wl_display_roundtrip)
 #define wl_proxy_destroy (*wfl_wl_proxy_destroy)
 #define wl_proxy_add_listener (*wfl_wl_proxy_add_listener)
+#define wl_proxy_set_user_data (*wfl_wl_proxy_set_user_data)
+#define wl_proxy_get_user_data (*wfl_wl_proxy_get_user_data)
+#define wl_proxy_get_version (*wfl_wl_proxy_get_version)
 #define wl_proxy_marshal (*wfl_wl_proxy_marshal)
 #define wl_proxy_marshal_constructor (*wfl_wl_proxy_marshal_constructor)
 #define wl_proxy_marshal_constructor_versioned (*wfl_wl_proxy_marshal_constructor_versioned)
