@@ -28,6 +28,7 @@
 
 #include <xcb/xcb.h>
 
+#include "wcore_attrib_list.h"
 #include "wcore_error.h"
 
 #include "wegl_config.h"
@@ -46,7 +47,7 @@ xegl_window_destroy(struct wcore_window *wc_self)
     if (!self)
         return ok;
 
-    ok &= wegl_window_teardown(&self->wegl);
+    ok &= wegl_surface_teardown(&self->wegl);
     ok &= x11_window_teardown(&self->x11);
     free(self);
     return ok;
@@ -55,32 +56,32 @@ xegl_window_destroy(struct wcore_window *wc_self)
 struct wcore_window*
 xegl_window_create(struct wcore_platform *wc_plat,
                    struct wcore_config *wc_config,
-                   int width,
-                   int height)
+                   int32_t width,
+                   int32_t height,
+                   const intptr_t attrib_list[])
 {
     struct xegl_window *self;
     struct xegl_display *dpy = xegl_display(wc_config->display);
     struct wegl_config *config = wegl_config(wc_config);
-    struct wegl_platform *plat = wegl_platform(wc_plat);
-    xcb_visualid_t visual;
     bool ok = true;
 
+    if (width == -1 && height == -1) {
+        width = DisplayWidth(dpy->x11.xlib, dpy->x11.screen);
+        height = DisplayHeight(dpy->x11.xlib, dpy->x11.screen);
+    }
+
+    if (wcore_attrib_list_length(attrib_list) > 0) {
+        wcore_error_bad_attribute(attrib_list[0]);
+        return NULL;
+    }
+
     self = wcore_calloc(sizeof(*self));
     if (self == NULL)
         return NULL;
 
-    ok = plat->eglGetConfigAttrib(dpy->wegl.egl,
-                                  config->egl,
-                                  EGL_NATIVE_VISUAL_ID,
-                                  (EGLint*) &visual);
-    if (!ok) {
-        wegl_emit_error(plat, "eglGetConfigAttrib(EGL_NATIVE_VISUAL_ID)");
-        goto error;
-    }
-
     ok = x11_window_init(&self->x11,
                          &dpy->x11,
-                         visual,
+                         (xcb_visualid_t) config->visual,
                          width,
                          height);
     if (!ok)
