@@ -118,6 +118,10 @@ typedef double              GLclampd;   /* double precision float in [0,1] */
 #define GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT 0x00000001
 #define GL_CONTEXT_FLAG_DEBUG_BIT              0x00000002
 
+#define GL_CONTEXT_PROFILE_MASK     0x9126
+#define GL_CONTEXT_CORE_PROFILE_BIT            0x00000001
+#define GL_CONTEXT_COMPATIBILITY_PROFILE_BIT   0x00000002
+
 #ifndef _WIN32
 #define APIENTRY
 #else
@@ -279,6 +283,50 @@ struct gl_basic_draw_args__ {
     bool alpha;
 };
 
+#define assert_int_equal_print(_a, _b) \
+    if (_a != _b) {\
+	fprintf(stderr, #_a " (%d) != " #_b "(%d)\n", _a, _b); \
+	assert_true(0 && "Integer comparison failed"); \
+    }
+
+#define assert_int_ge(_a, _b) \
+    if (_a < _b) {\
+	fprintf(stderr, #_a " (%d) < " #_b "(%d)\n", _a, _b); \
+	assert_true(0 && "Integer comparison failed"); \
+    }
+
+#define assert_in_range_print(_a, _min, _max) \
+    if (_a < _min || _a > _max) {\
+	fprintf(stderr, #_a " (%d) outside range " #_min "(%d) - " #_max "(%d)\n", \
+                _a, _min, _max); \
+	assert_true(0 && "Integer comparison failed"); \
+    }
+
+#define assert_string_len_equal(_actual, _expected, _len) \
+    if (strncmp(_actual, _expected, _len) != 0) {\
+	fprintf(stderr, "Expected (" #_expected "): \"%.*s\"\n", (int)_len, _expected); \
+	fprintf(stderr, "Received (" #_actual "): \"%.*s\"\n", (int)_len, _actual); \
+	assert_true(0 && "String comparison failed"); \
+    }
+
+static void
+error_waffle(void)
+{
+    const struct waffle_error_info *info = waffle_error_get_info();
+    const char *code = waffle_error_to_string(info->code);
+
+    if (info->message_length > 0)
+        fprintf(stderr, "Waffle error: 0x%x %s: %s\n", info->code, code, info->message);
+    else
+        fprintf(stderr, "Waffle error: 0x%x %s\n", info->code, code);
+}
+
+#define assert_true_with_wfl_error(_arg) \
+    if (!(_arg)) { \
+        error_waffle(); \
+        assert_true(0); \
+    }
+
 static void
 gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
 {
@@ -291,6 +339,7 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     bool context_debug = args.debug;
     bool context_robust = args.robust;
     bool alpha = args.alpha;
+    bool ret;
 
     int32_t config_attrib_list[64];
     int i;
@@ -337,7 +386,8 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     config_attrib_list[i++] = 0;
 
     // Create objects.
-    assert_true(ts->dpy = waffle_display_connect(NULL));
+    ts->dpy = waffle_display_connect(NULL);
+    assert_true_with_wfl_error(ts->dpy);
 
     ts->config = waffle_config_choose(ts->dpy, config_attrib_list);
     if (expect_error) {
@@ -353,12 +403,15 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
             // config flavor.
             skip();
         default:
-            assert_true(0);
+            assert_true_with_wfl_error(ts->config);
         }
     }
 
-    assert_true(ts->window = waffle_window_create2(ts->config, window_attrib_list));
-    assert_true(waffle_window_show(ts->window));
+    ts->window = waffle_window_create2(ts->config, window_attrib_list);
+    assert_true_with_wfl_error(ts->window);
+
+    ret = waffle_window_show(ts->window);
+    assert_true_with_wfl_error(ret);
 
     ts->ctx = waffle_context_create(ts->config, NULL);
     if (ts->ctx == NULL) {
@@ -370,7 +423,7 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
             // context flavor.
             skip();
         default:
-            assert_true(0);
+            assert_true_with_wfl_error(ts->ctx);
         }
     }
 
@@ -382,7 +435,8 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     assert_true(glReadPixels    = get_gl_symbol(waffle_context_api, "glReadPixels"));
     assert_true(glGetString     = get_gl_symbol(waffle_context_api, "glGetString"));
 
-    assert_true(waffle_make_current(ts->dpy, ts->window, ts->ctx));
+    ret = waffle_make_current(ts->dpy, ts->window, ts->ctx);
+    assert_true_with_wfl_error(ret);
 
     assert_true(waffle_get_current_display() == ts->dpy);
     assert_true(waffle_get_current_window() == ts->window);
@@ -411,20 +465,21 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     }
 
     const size_t version_str_len = strlen(expected_version_str);
-    assert_true(strncmp(version_str, expected_version_str, version_str_len) == 0);
+    assert_string_len_equal(version_str, expected_version_str, version_str_len);
     version_str += version_str_len;
 
     count = sscanf(version_str, "%d.%d", &major, &minor);
-    assert_true(count == 2);
-    assert_true(major >= 0);
-    assert_true(minor >= 0 && minor < 10);
+    assert_int_equal_print(count, 2);
+    assert_int_ge(major, 0);
+    assert_in_range_print(minor, 0, 10);
 
     if (context_version != WAFFLE_DONT_CARE) {
         int expected_major = context_version / 10;
         int expected_minor = context_version % 10;
 
-        assert_true(major >= expected_major);
-        assert_true(minor >= expected_minor || major > expected_major);
+        assert_int_ge(major, expected_major);
+        if (major == expected_major)
+           assert_int_ge(minor, expected_minor);
     }
 
     const char *profile_suffix = "";
@@ -433,9 +488,18 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
         switch (context_profile) {
         case WAFFLE_CONTEXT_CORE_PROFILE:
             profile_suffix = " (Core Profile)";
+#ifdef __APPLE__
+            fprintf(stderr, "MacOS Core contexts, omit the \"%s\" suffix in glGetString(GL_VERSION)."
+                            "Applying workaround.\n", profile_suffix);
+            profile_suffix = "";
+#endif
             break;
         case WAFFLE_CONTEXT_COMPATIBILITY_PROFILE:
-            profile_suffix = " (Compatibility Profile)";
+            // HACK: seems like Mesa 19.3.3 at least will report
+            if (context_forward_compatible)
+                profile_suffix = " (Core Profile)";
+            else
+                profile_suffix = " (Compatibility Profile)";
             break;
         case WAFFLE_DONT_CARE:
             break;
@@ -449,10 +513,34 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     sprintf(profile_str, "%d.%d%s", major, minor, profile_suffix);
 
     const size_t profile_str_len = strlen(profile_str);
-    assert_true(strncmp(version_str, profile_str, profile_str_len) == 0);
+    assert_string_len_equal(version_str, profile_str, profile_str_len);
 
     int version_10x = 10 * major + minor;
 
+    // Another profile check
+    if (waffle_context_api == WAFFLE_CONTEXT_OPENGL && version_10x >= 32) {
+        GLint profile_mask = 0;
+        glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &profile_mask);
+
+        switch (context_profile) {
+        case WAFFLE_CONTEXT_CORE_PROFILE:
+            assert_true(profile_mask & GL_CONTEXT_CORE_PROFILE_BIT);
+            break;
+        case WAFFLE_CONTEXT_COMPATIBILITY_PROFILE:
+            // HACK: seems like Mesa 19.3.3 at least will report
+            if (context_forward_compatible)
+                assert_true(profile_mask & GL_CONTEXT_CORE_PROFILE_BIT);
+            else
+                assert_true(profile_mask & GL_CONTEXT_COMPATIBILITY_PROFILE_BIT);
+            break;
+        case WAFFLE_DONT_CARE:
+            break;
+        default:
+            assert_true(0);
+            break;
+        }
+    }
+
     if ((waffle_context_api == WAFFLE_CONTEXT_OPENGL && version_10x >= 30) ||
         (waffle_context_api != WAFFLE_CONTEXT_OPENGL && version_10x >= 32)) {
         GLint context_flags = 0;
@@ -490,7 +578,8 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     ASSERT_GL(glReadPixels(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT,
                            GL_RGBA, GL_UNSIGNED_BYTE,
                            ts->actual_pixels));
-    assert_true(waffle_window_swap_buffers(ts->window));
+    ret = waffle_window_swap_buffers(ts->window);
+    assert_true_with_wfl_error(ret);
 
     assert_memory_equal(&ts->actual_pixels, &ts->expect_pixels,
                         sizeof(ts->expect_pixels));
@@ -616,6 +705,17 @@ static void test_gl_basic_gl##waffle_version##_compat(void **state)     \
                   .expect_error=WAFFLE_##error);                        \
 }
 
+#define test_glXX_compat_fwdcompat(waffle_version, error)               \
+static void test_gl_basic_gl##waffle_version##_compat_fwdcompat(void **state) \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .version=waffle_version,                              \
+                  .profile=WAFFLE_CONTEXT_COMPATIBILITY_PROFILE,        \
+                  .forward_compatible=true,                             \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
 #define test_glXX_compat_debug(waffle_version, error)                   \
 static void test_gl_basic_gl##waffle_version##_compat_debug(void **state) \
 {                                                                       \
@@ -675,7 +775,6 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gl15),                             \
         unit_test_make(test_gl_basic_gl20),                             \
         unit_test_make(test_gl_basic_gl21),                             \
-        unit_test_make(test_gl_basic_gl21_fwdcompat),                   \
                                                                         \
         unit_test_make(test_gl_basic_gl30),                             \
         unit_test_make(test_gl_basic_gl30_fwdcompat),                   \
@@ -691,8 +790,12 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gl41_core),                        \
         unit_test_make(test_gl_basic_gl42_core),                        \
         unit_test_make(test_gl_basic_gl43_core),                        \
+        unit_test_make(test_gl_basic_gl44_core),                        \
+        unit_test_make(test_gl_basic_gl45_core),                        \
+        unit_test_make(test_gl_basic_gl46_core),                        \
                                                                         \
         unit_test_make(test_gl_basic_gl32_compat),                      \
+        unit_test_make(test_gl_basic_gl32_compat_fwdcompat),            \
         unit_test_make(test_gl_basic_gl32_compat_debug),                \
         unit_test_make(test_gl_basic_gl32_compat_robust),               \
         unit_test_make(test_gl_basic_gl33_compat),                      \
@@ -700,6 +803,9 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gl41_compat),                      \
         unit_test_make(test_gl_basic_gl42_compat),                      \
         unit_test_make(test_gl_basic_gl43_compat),                      \
+        unit_test_make(test_gl_basic_gl44_compat),                      \
+        unit_test_make(test_gl_basic_gl45_compat),                      \
+        unit_test_make(test_gl_basic_gl46_compat),                      \
                                                                         \
         unit_test_make(test_gl_basic_gles1_rgb),                        \
         unit_test_make(test_gl_basic_gles1_rgba),                       \
@@ -722,14 +828,40 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gles3_debug),                      \
         unit_test_make(test_gl_basic_gles3_robust),                     \
         unit_test_make(test_gl_basic_gles30),                           \
+        unit_test_make(test_gl_basic_gles31),                           \
+        unit_test_make(test_gl_basic_gles32),                           \
                                                                         \
     };                                                                  \
                                                                         \
     return cmocka_run_group_tests_name(#platform, tests, NULL, NULL);   \
 }
 
+//
+// Most of the following tests will return ERROR_UNSUPPORTED_ON_PLATFORM
+// on Apple/CGL, where NO_ERROR is expected.
+// This is safe, as the test is skipped when the said error occurs.
+//
+// As BAD_ATTRIBUTE (core validation) takes greater precedence over
+// UNSUPPORTED_ON_PLATFORM (platform specific one).
+// Thus we're safe to use the former here, eventhough CGL has support
+// for neither GLES* nor fwdcompa "CGL and everyone else".
+//
+
+//
+// Quick notes which combinations we test and why
+// - w/o version, for each API/profile combination check variations of
+//   - visual - make sure we can draw fine, duh
+//   - flags - flags selection machinery is funky
+// - fwdcompat was introduced with OpenGL 3.0, check it and 3.1
+// - specific version
+//   - all known versions
+//   - no permutation
+//
 test_XX_rgb(gl, OPENGL, NO_ERROR)
 test_XX_rgba(gl, OPENGL, NO_ERROR)
+test_XX_fwdcompat(gl, OPENGL, ERROR_BAD_ATTRIBUTE)
+test_XX_debug(gl, OPENGL, NO_ERROR)
+test_XX_robust(gl, OPENGL, NO_ERROR)
 
 test_glXX(10, NO_ERROR)
 test_glXX(11, NO_ERROR)
@@ -739,19 +871,6 @@ test_glXX(14, NO_ERROR)
 test_glXX(15, NO_ERROR)
 test_glXX(20, NO_ERROR)
 test_glXX(21, NO_ERROR)
-test_glXX_fwdcompat(21, ERROR_BAD_ATTRIBUTE)
-
-
-//
-// Most of the following tests will return ERROR_UNSUPPORTED_ON_PLATFORM
-// on Apple/CGL, where NO_ERROR is expected.
-// This is safe, as the test is skipped when the said error occurs.
-//
-
-test_XX_fwdcompat(gl, OPENGL, ERROR_BAD_ATTRIBUTE)
-test_XX_debug(gl, OPENGL, NO_ERROR)
-test_XX_robust(gl, OPENGL, NO_ERROR)
-
 test_glXX(30, NO_ERROR)
 test_glXX_fwdcompat(30, NO_ERROR)
 test_glXX(31, NO_ERROR)
@@ -766,8 +885,12 @@ test_glXX_core(40, NO_ERROR)
 test_glXX_core(41, NO_ERROR)
 test_glXX_core(42, NO_ERROR)
 test_glXX_core(43, NO_ERROR)
+test_glXX_core(44, NO_ERROR)
+test_glXX_core(45, NO_ERROR)
+test_glXX_core(46, NO_ERROR)
 
 test_glXX_compat(32, NO_ERROR)
+test_glXX_compat_fwdcompat(32, NO_ERROR)
 test_glXX_compat_debug(32, NO_ERROR)
 test_glXX_compat_robust(32, NO_ERROR)
 test_glXX_compat(33, NO_ERROR)
@@ -775,9 +898,13 @@ test_glXX_compat(40, NO_ERROR)
 test_glXX_compat(41, NO_ERROR)
 test_glXX_compat(42, NO_ERROR)
 test_glXX_compat(43, NO_ERROR)
+test_glXX_compat(44, NO_ERROR)
+test_glXX_compat(45, NO_ERROR)
+test_glXX_compat(46, NO_ERROR)
 
 test_XX_rgb(gles1, OPENGL_ES1, NO_ERROR)
 test_XX_rgba(gles1, OPENGL_ES1, NO_ERROR)
+test_XX_fwdcompat(gles1, OPENGL_ES1, ERROR_BAD_ATTRIBUTE)
 test_XX_debug(gles1, OPENGL_ES1, NO_ERROR)
 test_XX_robust(gles1, OPENGL_ES1, NO_ERROR)
 test_glesXX(1, 10, NO_ERROR)
@@ -785,39 +912,20 @@ test_glesXX(1, 11, NO_ERROR)
 
 test_XX_rgb(gles2, OPENGL_ES2, NO_ERROR)
 test_XX_rgba(gles2, OPENGL_ES2, NO_ERROR)
+test_XX_fwdcompat(gles2, OPENGL_ES2, ERROR_BAD_ATTRIBUTE)
 test_XX_debug(gles2, OPENGL_ES2, NO_ERROR)
 test_XX_robust(gles2, OPENGL_ES2, NO_ERROR)
 test_glesXX(2, 20, NO_ERROR)
 
 test_XX_rgb(gles3, OPENGL_ES3, NO_ERROR)
 test_XX_rgba(gles3, OPENGL_ES3, NO_ERROR)
+test_XX_fwdcompat(gles3, OPENGL_ES3, ERROR_BAD_ATTRIBUTE)
 test_XX_debug(gles3, OPENGL_ES3, NO_ERROR)
 test_XX_robust(gles3, OPENGL_ES3, NO_ERROR)
 test_glesXX(3, 30, NO_ERROR)
+test_glesXX(3, 31, NO_ERROR)
+test_glesXX(3, 32, NO_ERROR)
 
-//
-// As BAD_ATTRIBUTE takes greater precedence over UNSUPPORTED_ON_PLATFORM,
-// we have to split the ESx + fwdcompat tests into "CGL and everyone else".
-//
-
-#if defined(WAFFLE_HAS_GBM) || \
-    defined(WAFFLE_HAS_GLX) || \
-    defined(WAFFLE_HAS_SURFACELESS_EGL) || \
-    defined(WAFFLE_HAS_WAYLAND) || \
-    defined(WAFFLE_HAS_X11_EGL) || \
-    defined(WAFFLE_HAS_WGL)
-
-test_XX_fwdcompat(gles1, OPENGL_ES1, ERROR_BAD_ATTRIBUTE)
-test_XX_fwdcompat(gles2, OPENGL_ES2, ERROR_BAD_ATTRIBUTE)
-test_XX_fwdcompat(gles3, OPENGL_ES3, ERROR_BAD_ATTRIBUTE)
-
-#elif defined(WAFFLE_HAS_CGL)
-
-test_XX_fwdcompat(gles1, OPENGL_ES1, ERROR_UNSUPPORTED_ON_PLATFORM)
-test_XX_fwdcompat(gles2, OPENGL_ES2, ERROR_UNSUPPORTED_ON_PLATFORM)
-test_XX_fwdcompat(gles3, OPENGL_ES3, ERROR_UNSUPPORTED_ON_PLATFORM)
-
-#endif
 
 #ifdef WAFFLE_HAS_CGL
 
@@ -861,7 +969,7 @@ CREATE_TESTSUITE(WAFFLE_PLATFORM_SURFACELESS_EGL, surfaceless_egl)
 
 #undef unit_test_make
 
-#endif // WAFFLE_HAS_GBM
+#endif // WAFFLE_HAS_SURFACELESS_EGL
 
 #ifdef WAFFLE_HAS_WAYLAND
 
@@ -900,6 +1008,7 @@ CREATE_TESTSUITE(WAFFLE_PLATFORM_WGL, wgl)
 
 #undef test_glXX_compat_robust
 #undef test_glXX_compat_debug
+#undef test_glXX_compat_fwdcompat
 #undef test_glXX_compat
 #undef test_glXX_core_robust
 #undef test_glXX_core_debug
